"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const path_1 = require("path");
const api_1 = __importDefault(require("@mesg/api"));
const orchestrator_1 = __importDefault(require("@mesg/orchestrator"));
const execution_1 = require("@mesg/orchestrator/lib/execution");
const listr_1 = __importDefault(require("listr"));
const dotenv_1 = __importDefault(require("dotenv"));
const Environment = __importStar(require("../utils/environment-tasks"));
const Service = __importStar(require("../utils/service"));
const Runner = __importStar(require("../utils/runner"));
const Process = __importStar(require("../utils/process"));
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const version_1 = __importDefault(require("../version"));
const chalk_1 = __importDefault(require("chalk"));
const encoder_1 = require("@mesg/orchestrator/lib/encoder");
const sign_1 = __importDefault(require("../utils/sign"));
const ipfsClient = require('ipfs-http-client');
class Dev extends command_1.Command {
    constructor() {
        super(...arguments);
        this.lcdEndpoint = 'http://localhost:1317';
        this.orchestratorEndpoint = 'localhost:50052';
        this.lcd = new api_1.default(this.lcdEndpoint);
        this.ipfsClient = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
        this.orchestrator = new orchestrator_1.default(this.orchestratorEndpoint);
        this.services = [];
        this.processes = [];
        this.runners = [];
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Dev);
            if (!fs_1.existsSync(path_1.join(args.PATH, 'services')))
                this.error(`${args.PATH} is not a recognized as a MESG project`);
            const envs = fs_1.existsSync(path_1.join(args.PATH, '.env'))
                ? dotenv_1.default.parse(fs_1.readFileSync(path_1.join(args.PATH, '.env')))
                : {};
            const env = Object.keys(envs).reduce((prev, x) => [...prev, `${x}=${envs[x]}`], []);
            const tasks = new listr_1.default([
                Environment.start
            ]);
            const serviceDirs = fs_1.readdirSync(path_1.join(args.PATH, 'services'), { withFileTypes: true })
                .filter(x => x.isDirectory() || x.isSymbolicLink());
            for (const dir of serviceDirs) {
                tasks.add([
                    {
                        title: `Creating service "${dir.name}"`,
                        task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                            const definition = yield Service.compile(path_1.join(args.PATH, 'services', dir.name), this.ipfsClient);
                            const service = yield Service.create(this.lcd, definition, ctx.config.mnemonic);
                            this.services.push(service);
                        })
                    }
                ]);
            }
            const processFiles = fs_1.readdirSync(args.PATH, { withFileTypes: true })
                .filter(x => x.isFile() && x.name.match(/\.(yml|yaml)/));
            for (const file of processFiles) {
                tasks.add({
                    title: `Creating process "${file.name}"`,
                    task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                        const compilation = yield Process.compile(path_1.join(args.PATH, file.name), this.ipfsClient, this.lcd, this.lcdEndpoint, this.orchestratorEndpoint, ctx.config.mnemonic, ctx.engineAddress, env);
                        const deployedProcess = yield Process.create(this.lcd, compilation.definition, ctx.config.mnemonic);
                        this.processes.push(deployedProcess);
                        this.runners = [
                            ...this.runners,
                            ...compilation.runners,
                        ].filter((item, i, self) => i === self.indexOf(item));
                    })
                });
            }
            tasks.add({
                title: 'Fetching logs',
                task: ctx => {
                    const payload = {
                        filter: {
                            statuses: [
                                execution_1.Status.Completed,
                                execution_1.Status.Failed
                            ]
                        }
                    };
                    this.logs = this.orchestrator.execution.stream(payload, sign_1.default(payload, ctx.config.mnemonic));
                }
            });
            const { config, engineAddress } = yield tasks.run({
                configDir: this.config.dataDir,
                endpoint: this.lcdEndpoint,
                pull: flags.pull,
                version: flags.version
            });
            this.logs
                .on('error', (error) => { this.warn('Result stream error: ' + error.message); })
                .on('data', (execution) => {
                if (!execution.processHash)
                    return;
                const process = this.processes.find(x => x.hash === base58.encode(execution.processHash));
                if (!process)
                    return;
                const prefix = `[${process.name}][${execution.nodeKey}] - ${base58.encode(execution.instanceHash)} - ${execution.taskKey}`;
                if (execution.error) {
                    this.log(`${prefix}: ` + chalk_1.default.red('ERROR:', execution.error));
                }
                if (!execution.outputs)
                    return;
                return this.log(prefix +
                    '\n\tinputs:  ' + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.inputs || {}))) +
                    '\n\toutputs: ' + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.outputs || {}))) +
                    '\n');
            });
            process.once('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                yield new listr_1.default([
                    {
                        title: 'Stopping logs',
                        task: () => {
                            if (this.logs)
                                this.logs.cancel();
                        }
                    },
                    {
                        title: 'Stopping running services',
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            const uniqueRunners = this.runners.filter((item, i, self) => i === self.indexOf(item));
                            for (const runner of uniqueRunners) {
                                yield Runner.stop(this.lcdEndpoint, this.orchestratorEndpoint, config.mnemonic, engineAddress, runner.hash);
                            }
                        })
                    },
                    {
                        title: 'Deleting processes',
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            for (const process of this.processes) {
                                yield Process.remove(this.lcd, process, config.mnemonic);
                            }
                        })
                    },
                    Environment.stop
                ]).run({
                    configDir: this.config.dataDir
                });
            }));
        });
    }
}
exports.default = Dev;
Dev.description = 'Start a dev environment for your project';
Dev.flags = {
    version: command_1.flags.string({ name: 'Engine version', default: version_1.default.engine }),
    pull: command_1.flags.boolean({ name: 'Force to pull the docker image', default: false }),
};
Dev.args = [{
        name: 'PATH',
        description: 'Path of your project',
        default: './'
    }];
