"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const api_1 = __importDefault(require("@mesg/api"));
const encoder_1 = require("@mesg/orchestrator/lib/encoder");
const listr_1 = __importDefault(require("listr"));
const resolve_1 = require("@mesg/api/lib/util/resolve");
const input_1 = require("../../utils/input");
const orchestrator_1 = __importDefault(require("@mesg/orchestrator"));
const execution_1 = require("@mesg/orchestrator/lib/execution");
const sign_1 = __importDefault(require("../../utils/sign"));
const uuid_1 = __importDefault(require("uuid"));
const config_1 = require("../../utils/config");
const struct_1 = require("@mesg/api/lib/struct");
const json_1 = __importDefault(require("cli-ux/lib/styled/json"));
class Execute extends command_1.Command {
    constructor() {
        super(...arguments);
        this.lcd = new api_1.default(`http://localhost:1317`);
        this.orchestrator = new orchestrator_1.default(`localhost:50052`);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Execute);
            const id = uuid_1.default();
            let runner;
            let task;
            let inputs;
            let logs;
            let execution;
            const config = config_1.generateConfig(this.config.dataDir);
            const tasks = new listr_1.default([
                {
                    title: 'Validating task',
                    task: () => __awaiter(this, void 0, void 0, function* () {
                        try {
                            runner = yield this.lcd.runner.get(args.RUNNER_HASH);
                        }
                        catch (err) {
                            const hash = yield resolve_1.resolveSIDRunner(this.lcd, args.RUNNER_HASH);
                            runner = yield this.lcd.runner.get(hash);
                        }
                        const instance = yield this.lcd.instance.get(runner.instanceHash);
                        const service = yield this.lcd.service.get(instance.serviceHash);
                        task = service.tasks.find(x => x.key === args.TASK);
                        if (!task)
                            this.error(`The task ${args.TASK} does not exist in service '${service.hash}'`);
                    })
                },
                {
                    title: 'Converting inputs',
                    task: () => {
                        inputs = input_1.convert(task, flags.json
                            ? JSON.parse(fs_1.readFileSync(flags.json).toString())
                            : (flags.data || []).reduce((acc, item) => (Object.assign(Object.assign({}, acc), { [item.split('=')[0]]: item.split('=')[1] })), {}));
                    }
                },
                {
                    title: 'Listening for result',
                    task: () => __awaiter(this, void 0, void 0, function* () {
                        const payload = {
                            filter: {
                                executorHash: runner.hash,
                                instanceHash: runner.instanceHash,
                                tags: [id],
                                taskKey: args.TASK,
                                statuses: [execution_1.Status.Completed, execution_1.Status.Failed]
                            }
                        };
                        logs = yield this.orchestrator.execution.stream(payload, sign_1.default(payload, config.mnemonic));
                    })
                },
                {
                    title: 'Executing task',
                    task: () => __awaiter(this, void 0, void 0, function* () {
                        const payload = {
                            executorHash: runner.hash,
                            inputs: inputs,
                            taskKey: args.TASK,
                            price: '10000atto',
                            tags: [id]
                        };
                        yield this.orchestrator.execution.create(payload, sign_1.default(Object.assign(Object.assign({}, payload), { inputs: struct_1.toStruct(payload.inputs) }), config.mnemonic));
                        return new Promise((resolve, reject) => {
                            logs.once('data', (exec) => {
                                execution = exec;
                                execution.status === execution_1.Status.Completed ? resolve(execution) : reject(new Error(execution.error));
                            });
                        });
                    })
                }
            ]);
            yield tasks.run();
            json_1.default(encoder_1.decode(execution.outputs));
            process.exit(0);
        });
    }
}
exports.default = Execute;
Execute.description = 'Execute a task on a running service';
Execute.flags = {
    json: command_1.flags.string({ char: 'j', description: 'Path to a JSON file containing the task inputs' }),
    data: command_1.flags.string({
        char: 'd',
        description: 'Task inputs',
        multiple: true,
        helpValue: 'key=value'
    }),
    eventHash: command_1.flags.string({
        description: 'Event hash to create the execution with'
    }),
};
Execute.args = [{
        name: 'RUNNER_HASH',
        required: true,
        description: 'The hash of the runner that will execute this execution'
    }, {
        name: 'TASK',
        required: true,
        description: 'Task key'
    }];
