"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const compiler_1 = require("@mesg/compiler");
const fs_1 = require("fs");
const Service = __importStar(require("./service"));
const Runner = __importStar(require("./runner"));
const txevent_1 = require("@mesg/api/lib/util/txevent");
const error_1 = require("./error");
exports.compile = (processFilePath, ipfsClient, lcd, lcdEndpoint, orchestratorEndpoint, mnemonic, engineAddress, env = []) => __awaiter(void 0, void 0, void 0, function* () {
    const runners = [];
    const instanceReolver = (instanceObject) => __awaiter(void 0, void 0, void 0, function* () {
        if (!instanceObject.instanceHash && !instanceObject.instance)
            throw new Error('"instanceHash" or "instance" not found in the process\'s definition');
        if (instanceObject.instanceHash)
            return instanceObject.instanceHash;
        const { src, env } = instanceObject.instance;
        const definition = yield Service.compile(src, ipfsClient);
        const service = yield Service.create(lcd, definition, mnemonic);
        const runner = yield Runner.create(lcdEndpoint, orchestratorEndpoint, mnemonic, engineAddress, service.hash, env);
        if (!runners.find(x => x.hash === runner.hash))
            runners.push(runner);
        return runner.instanceHash;
    });
    const definition = yield compiler_1.process(fs_1.readFileSync(processFilePath), instanceReolver, env.reduce((prev, env) => (Object.assign(Object.assign({}, prev), { [env.split('=')[0]]: env.split('=')[1] })), {}));
    return {
        definition,
        runners
    };
});
exports.create = (lcd, process, mnemonic) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const account = yield lcd.account.import(mnemonic);
        const tx = yield lcd.createTransaction([lcd.process.createMsg(account.address, process)], account);
        const txResult = yield lcd.broadcast(tx.signWithMnemonic(mnemonic));
        const hashes = txevent_1.findHash(txResult, "Process");
        if (hashes.length != 1)
            throw new Error('error while getting the hash of the process created');
        return lcd.process.get(hashes[0]);
    }
    catch (e) {
        if (!error_1.IsAlreadyExistsError.match(e))
            throw e;
        const hash = new error_1.IsAlreadyExistsError(e).hash;
        return lcd.process.get(hash);
    }
});
exports.remove = (lcd, process, mnemonic) => __awaiter(void 0, void 0, void 0, function* () {
    const account = yield lcd.account.import(mnemonic);
    const tx = yield lcd.createTransaction([lcd.process.deleteMsg(account.address, process.hash)], account);
    yield lcd.broadcast(tx.signWithMnemonic(mnemonic));
    return;
});
