"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const deployer_1 = __importStar(require("./deployer"));
const fs_1 = require("fs");
const path_1 = require("path");
const compiler_1 = require("@mesg/compiler");
const txevent_1 = require("@mesg/api/lib/util/txevent");
exports.compile = (path, ipfsClient) => __awaiter(void 0, void 0, void 0, function* () {
    const deployedPath = yield deployer_1.default(path);
    const service = yield compiler_1.service(fs_1.readFileSync(path_1.join(deployedPath, 'mesg.yml')));
    const buffer = [];
    service.source = yield new Promise((resolve, reject) => {
        deployer_1.createTar(path_1.join(deployedPath))
            .on('data', (data) => buffer.push(...data))
            .once('error', reject)
            .on('end', () => __awaiter(void 0, void 0, void 0, function* () {
            const res = yield ipfsClient.add(Buffer.from(buffer), { pin: true });
            if (!res.length) {
                return reject(new Error('pushing service manifest failed'));
            }
            return resolve(res[0].hash);
        }));
    });
    return service;
});
exports.create = (lcd, definition, mnemonic) => __awaiter(void 0, void 0, void 0, function* () {
    const hash = yield lcd.service.hash(definition);
    if (yield lcd.service.exists(hash))
        return lcd.service.get(hash);
    const account = yield lcd.account.import(mnemonic);
    const tx = yield lcd.createTransaction([lcd.service.createMsg(account.address, definition)], account);
    const txResult = yield lcd.broadcast(tx.signWithMnemonic(mnemonic));
    const hashes = txevent_1.findHash(txResult, "Service");
    if (hashes.length != 1)
        throw new Error('error while getting the hash of the service created');
    return lcd.service.get(hashes[0]);
});
